## @summary Common functions for data collection scripts
##
## @description
##   Provide common support functions for the various info scripts in
##   the attach kit.
##
##
# Copyright (C) 2002 - 2008 NetApp, Inc.

package		SHsupport;
require		Exporter;
@ISA 		= qw(Exporter);
@EXPORT   	= qw();
use strict;
use Cwd;
use Net::Ping;
use English;
use File::Path;

my ($directory,  # Output directory for stuff
    $subdir,     # Name of subdir in directory, e.g., "ntap_filer_slug"
    $tar,	 # Path to tar binary
    $tarfile,	 # The name of the tar file
    $gzfile,     # Name of gzipped tar file
    $zfile,      # Name of compress tar file
    $tgzfile,    # Alternate name of gzipped tar file
    $compress,   # Path to compress binary
    $rsh, 	 # Path to rsh binary
    $useperltar, # Set if we use internal Perl tar commands
    $tarball,	 # Name of final tarball
    $no_files,	 # Do no file output
    $colcount,   # Column number for user output
    @filelist,   # List of files to be added to archive
);
$colcount = 0;

## @summary setup OS specific items
##
## @description
##
## Takes an optional argument that specifies an alternative output
## directory. In absence of argument the default path for the OS is set.
## Sets paths to rsh, tar, whoami, and compress. Sets flag for use of 
## internal Perl tar functions for Windows
##
## @synopsys get_os_specifc [altdir]
##
## @arg altdir Optional: Alternate directory for output
##
## @usage
sub get_os_specific {
    my ($altdir) = @_;
    $_ = $OSNAME;
    $directory = "/tmp/ontap";
  SWITCH: {
	/MSWin32/ && do {
	    $directory = "\\temp\\ontap";
	    $rsh = "rsh";
	    $useperltar = 1;
		last SWITCH;
	};
	/linux/ && do {
	    ##  Linux comes with gzip and gzip does a much
	    ##  better job compressing than compress, so
	    ##  why doesn't Linux use gzip as well?
	    $rsh = "/usr/bin/rsh";
	    $tar = "/bin/tar";
	    $compress = "/usr/bin/gzip";
		last SWITCH;
	};
	/solaris/ && do {
	    $rsh = "/usr/bin/rsh";
	    $tar = "/usr/sbin/tar";
	    $compress = "/usr/bin/compress";
		last SWITCH;
	};
	/hpux/ && do {
	    $rsh = "/usr/bin/remsh";
	    $tar = "/usr/bin/tar";
	    $compress = "/usr/contrib/bin/gzip -9";
		last SWITCH;
	};
	/aix/ && do {
	    $rsh = "/usr/bin/rsh";
	    $tar = "/usr/bin/tar";
	    $compress = "/usr/bin/compress";
		last SWITCH;
	};
    }
    if (defined($altdir)) {
	$directory = $altdir;
    }
}

## @summary
##
## @description
##   Test for the existence of the output data directory,
##   intermediate tar file, or either flavor of compressed output in
##   the output directory. If found, it complains about it, instructs
##   the user to delete or rename, and exits. If OK, it creates the
##   data output directory and chdir()s there.
##
## @synopsys
##   test_existing_files {subdir}
##
## @arg subdir Required: directory in which to test/create output. This
##   is below "$directory".
## @arg suppress Optional: if defined, do not do directory checking
##   Also, flag make_tarball and print_summary to do nothing.
## @usage
##
sub test_existing_files {
    ($subdir, my $suppress, my $force) = @_;

    if (defined($suppress) && $suppress == 1) {
	$no_files = 1;
	return;
    }

    if (! -d $directory) {
	mkpath($directory,0,0777) or die "Unable to mkdir $directory";
    }
    
    my $olddir = getcwd();
    chdir($directory) or die "Unable to chdir to $directory";

    $tarfile = $subdir . ".tar";
    $gzfile = $tarfile . ".gz";
    $zfile = $tarfile . ".Z";
    ($tgzfile = $tarfile . "tgz" ) =~ s/tar//; 

    my $of;
    if (defined($force)) {
	unlink $tarfile;
	unlink $gzfile;
	unlink $zfile;
	unlink $tgzfile;
	rmtree ($subdir, 0, 1);
    }
    foreach $of ($subdir, $tarfile, $gzfile, $zfile, $tgzfile) {
	if (-f $of || -d $of) {
            if ($OSNAME eq "MSWin32") {
	      print "$directory\\$of already exists; please delete or rename it.\n";
            } else {
              print "$directory/$of already exists; please delete or rename it.\n";
            }
	    exit 1;
	}
    }

    
    # Create the output directory and chdir there
    mkdir($subdir,0777) or die "Unable to mkdir $subdir";
    chdir($subdir) or die "Unable to chdir to $directory";
}



## @summary Tar a directory recursively
##
## @description
## Recursively descend a directory, adding files/directories
## Used for internal Perl tarring
##
## @synopsys
##   tardir {atar} {dir}
##
## @arg atar Required: tar object in use for building archive
## @arg dir Required: directory to traverse
##
## @usage
##
sub tardir {
    ##my ($atar, $dir) = @_;
    my ($dir) = @_;
    my $i = 0; 
    local *DIRH;
    opendir DIRH, $dir or die "Unable to opendir $dir";
    while (defined(my $f = readdir DIRH)) {
	my $path = $dir . "/" . $f;
	next if $f eq ".";
	next if $f eq "..";
	## $atar->add_files($path);
        $filelist[$i] = $path; 
        $i++;
	if (-d $path) {
	    ##tardir($atar, $path);
            tardir($path);
	}
    }
    closedir DIRH;
}

## @summary Create a tarball from the output directory
##
## @description
##   Make a tarball. It can be produced by external tar and compress
##   or (for Windows) Perl internal tar and gzip modules. Uses previously
##   defined directory and file suffices.
##
## @synopsys
##   make_tarball
##
## @usage
##
sub make_tarball {
    return if $no_files;
    if (defined($useperltar)) {
        require Archive::Tar;
	## my $atar = Archive::Tar->new();
	## tardir($atar, ".");
	tardir(".");
        my $atar = Archive::Tar->create_archive("../$gzfile", 2, @filelist);
	## $atar->write("../$gzfile", 1);
	$tarball = $gzfile;
    } else {
	my $retval;

	chdir "..";
	system("$tar -cf $tarfile $subdir");
	$retval = $? >> 8;
	if (($? >> 8) != 0) {
	    die "tar failed with status $retval";
	}
	system("$compress $tarfile");
	if (($? >> 8) != 0) {
	    die "compress failed with status $retval";
	}
	if ( $OSNAME eq "hpux" | $OSNAME eq "linux" ) {
	    $tarball = $gzfile;
	} else {
	    $tarball = $zfile;
	}
	chdir $subdir or die "Unable to chdir to $subdir";
    }

    return $tarball;
}

#
# Rsh commands - used for filer_info
#

## @summary Send rsh command and store output in file
##
## @description
##   Send a command string. Then gather all the output until the
##   prompt and writes it to a file of the same name as the command 
##   in the current directory.
##
## @synopsys
##   rshgetother {target} {user} {cmd} {outfile}
##
## @arg target Required: remote host
## @arg user Required: user/password combo
## @arg cmd Required: command to execute on remote host
## @arg outfile Required: file to collect output
##
## @usage
##
sub rshgetother {
    my ($target, $user, $cmd, $outf) = @_;
    my $line = rshget($target, $user, $cmd);
    if ($? != 0) {
	return 0;
    }
    open OUTFILE, ">$outf" or die "Unable to open file $outf for write";
    print OUTFILE $line;
    close OUTFILE;
    return 1;
}

## @summary Send rsh command and return output string
##
## @description
##   Send a command string. Then gather all the output and return it to
##   the caller.
##
## @synopsys
##   rshget {target} {user} {cmd}
##
## @arg target Required: remote host
## @arg user Required: user/password combo
## @arg cmd Required: command to execute on remote host
##
## @usage
##
sub rshget {
    my ($target, $user, $cmd) = @_;
    return `$rsh $target -l $user \"$cmd\" 2>&1`;
}

## @summary test for invoker having root-equivalence
##
## @description
##  Check for EUID of 0 for Unix, or user "Adminstrator" for Win2k.
##
## @synopsys
##  check_root_status
##
## @arg
##
## @usage
##
sub check_root_status {
    if ($OSNAME ne "MSWin32") {
	if ($EUID != 0) {
	    print "You must be root to run this script. (current uid: $EUID)\n";
	    exit 1;
	}
    } else {
# Disable check for Windows until we find a way to make NATE work
#	my $config = `net config workstation`;
#	$config =~ /User name\s*(\S*)/;
#	if ($1 ne "Administrator") {
#	    print "You must be Administrator to run this command (current $1)\n";
#	    exit 1;
#	}
    }
}

## @summary print final output message
##
## @description
## Print the boilerplate output that comes out after a successful info
## script run. 
##
## @synopsys
##   print_summary {string}
##
## @arg string Required: the first phrase in the output, e.g.,
##   "Filer lettuce" or "Switch brcd1".
##
## @usage
##
sub print_summary {
    return if $no_files;
    my ($hdr) = @_;
    my $here = getcwd();
    
    if ($OSNAME eq "MSWin32") {
	$here =~ s,/,\\,g;
    }
    print "\n$hdr info is in directory $here\n";
    chdir "..";
    my $there = getcwd() . "/$tarball";
    if ($OSNAME eq "MSWin32") {
	$there =~ s,/,\\,g;
    }
    
    print "\tCompressed file is $there\n";
    print "\tPlease send this file to Customer Support for analysis.\n";
}

##
## Ranga: routine for printing IBM specific messages
## TODO: The ideal way appears to pass the string and the vendor name and
## but i am not sure if i change the parameters to print_summary
## if and how it will affect the other info scripts. 
##
##
## @summary print final output message
##
## @description
## Print the boilerplate output that comes out after a successful info
## script run. 
##
## @synopsys
##   print_summary_ibm {string}
##
## @arg string Required: the first phrase in the output, e.g.,
##   "Filer lettuce" or "Switch brcd1".
##
## @usage
##
sub print_summary_ibm {
    return if $no_files;
    my ($hdr) = @_;
    my $here = getcwd();
    
    if ($OSNAME eq "MSWin32") {
	$here =~ s,/,\\,g;
    }
    print "\n$hdr info is in directory $here\n";
    chdir "..";
    my $there = getcwd() . "/$tarball";
    if ($OSNAME eq "MSWin32") {
	$there =~ s,/,\\,g;
    }
    
    print "\tCompressed file is $there\n";
    print "\tPlease send this file to IBM Customer Support for analysis.\n";
}

## @summary Check if the VMware Firewall ports have been opened.
##
## @description
##   This will check if the VMware Firewall ports are open.  If they are
##   not, it will exit with an appropriate error message.
##
##   IF the OSNAME is not equal to VMware, then it will immediately return success.
##
## @synopsys
##   check_firewall_status ()
##
## @arg NONE
## @usage
##
sub check_firewall_status {
    $_ = $OSNAME;
    my $firewall = "/usr/sbin/esxcfg-firewall";
    my $firewall_rc = -1;
    if ($OSNAME eq "vmware_os")
    {
       # Do one more sanity check to make sure this only executes on VMware ESX
       if ( -e $firewall )
       {
          system("$firewall -q ehuFirewall > /dev/null 2>&1");
             # For Perl 'system' calls, the real return code needs to be divided by 256;
             $firewall_rc = ($? / 256);
             if ($firewall_rc != 0)
             {
               if ($firewall_rc == 1)
               {
               print "The ehuFirewall firewall service is not enabled. Please run\n";
               print "\t$firewall -e ehuFirewall\n";
                  print "to enable the service and then try the command again.\n";
                  exit 1;
               } 
               else
               {
               # All others errors...
               print "The ehuFirewall firewall service does not exist. Please reinstall the\n"; 
                     print "Host Utilities and then try the command again.\n";
                     exit 1;
                  }
               }
       }
    }
}

## @summary Attempt to ping a remote node. Tries for 5 seconds.
##
## @description
##   Create a Ping object for the remote node and attempt an ICMP ping
##   for five seconds. If this succeeds, just return. If it fails, print
##   out a failure message and exit.
##
## @synopsys
##   ping_remote {type} {host}
##
## @arg type Required: Description of remote host, e.g., "host", "filer"
## @arg host Required: remote host name
##
## @usage
##
sub ping_remote {
    my ($type, $host) = @_;

    my $p = Net::Ping->new("icmp");

    if (!($p->ping($host, 5))) {
	print "Unable to reach $type $host\n";
	exit 2;
    }
}

## @summary Attempt to tcp ping the telnet port of a remote node.
##
## @description
##   Create a ping object for the remote node and attempt a TCP ping to
##   the telnet (23) port for five seconds. If this succeeds, just return. 
##   If it fails, print out a failure message and exit.
##
## @synopsys
##   telnet_ping_remote {type} {host}
##
## @arg type Required: Description of remote host, e.g., "host", "filer"
## @arg host Required: remote host name
##
## @usage
##
sub telnet_ping_remote {
    my ($type, $host) = @_;

	# Disable ping on Windows
	if ($OSNAME eq "MSWin32") {
		return 0;
	}

    my $p = Net::Ping->new("tcp");
    $p->{port_num} = 23;

    if (!($p->ping($host, 5))) {
	print "Unable to reach $type $host\n";
	exit 1;
    }
}

## @summary Keep outputting stuff to the console while we work 
##
## @description Print a dot to the output for the user to show that things
##   are happening while we work. Outputs them until
##   80 columns wrap, then inserts newline and keeps going.
##
## @synopsis
##    update_user
##
sub update_user {
    $colcount ++;
    if ($colcount >= 80) {
	print STDERR "\n";
	$colcount = 0;
    }   
    print STDERR ".";
}

## @summary Send a command through telnet and return response, even on Brocade
##
## @description
## Basic function that sends a command through the Telnet object, executes
## that command and effects/modifies the output scalar passed as a reference
## 
## @synopsys
##    getit {tlnt} {cmd} {out} {remote} {user}
##
## @arg tlnt Required: Telnet object for connection. Assumed that connection
## is already established.
##
## @arg cmd Required: Command to execute
##
## @arg out Required: Reference to scalar to receive output text
##
## @arg user Required: Name of user
##
sub getit {
    my ($tlnt, $cmd, $data_reference, $normal_prompt) = @_;

    my ($block);

    $tlnt->prompt("/$normal_prompt/");

    # add alarm (10 minutes) so process of running command
    # and reaping the results is constrained
    $SIG{ALRM} = sub { die "TIMEOUT OCCURRED\n"; };

    eval {
        # set alarm at 10 minutes
        alarm(600);

        # start the collection process, run in binary mode
        # and read block at a time (use default blocksize)
        $tlnt->binmode(1);
        $tlnt->print($cmd);

        # read until we reach the prompt or until there is
        # nothing left to read. Add the read data to the
        # reference scalar that is effected by this function
        while (($block = $tlnt->get()) and $block !~ /$normal_prompt/o) {
            $$data_reference .= $block;
        }
        # add the last block to the buffer
        $$data_reference .= $block;

        # reset alarm
        alarm(0);
    };

    # check to see if we timed out (from alarm)
    if ($@ =~ /TIMEOUT OCCURRED/) {
        print STDERR "\nError: TIMED OUT running command ($cmd)\n";
        print STDERR "       Only only partial data maybe returned\n";
    }

    $$data_reference =~ s/\r//g;

}


1;
